using VL.TestFramework;

namespace tests;

public class Tests
{
    // The directory of this dll
    public static string ThisDirectory => Path.GetDirectoryName(typeof(Tests).Assembly.Location)!;
    // Assuming the default output location the root dir of our package should be
    public static string PackageDirectory => Path.GetFullPath(Path.Combine(ThisDirectory, "..", "..", "..", ".."));
    // Path to our help patches
    public static string HelpDirectory => Path.Combine(PackageDirectory, "help");
    // We pass this one as a search path to vvvv
    public static string PackageRepository => Path.GetFullPath(Path.Combine(PackageDirectory, ".."));

    TestEnvironment? testEnvironment;

    // Watch out: Don't use async Task here. NUnit keeps a synchronization context per async method,
    // subsequent calls to the session therefor fail with the exception that the context had been shut down.
    // See https://github.com/nunit/nunit/issues/3500
    [OneTimeSetUp]
    public void Setup()
    {
        // Sadly the auto version selection with the 5.2 release doesn't work, so we need to specify the build id and version manually so the download works
        var vvvvExePath = Task.Run(() => TestEnvironmentLoader.DownloadEntryAssemblyAsync(37342, "5.2"));
        // You might want to pass in the proper search path here so your package is found
        testEnvironment = TestEnvironmentLoader.Load(vvvvExePath.Result!, searchPaths: new [] { PackageRepository });
    }

    [OneTimeTearDown]
    public void TearDown()
    {
        testEnvironment?.Dispose();
        testEnvironment = null;
    }

    [TestCaseSource(nameof(GetHelpPatches))]
    public async Task IsntRed(string filePath)
    {
        // Note: If runEntryPoint = true, we need to add <UseWindowsForms>true</UseWindowsForm> to the tests.csproj file, otherwise we get a weired crash on startup
        await testEnvironment!.LoadAndTestAsync(filePath, runEntryPoint: false);
    }

    public static IEnumerable<string> GetHelpPatches()
    {
        // Here you would specify what patches should be looked at
        return Directory.EnumerateFiles(HelpDirectory, "*.vl");
    }
}